
# -*- encoding:utf-8 -*-
# 経理システム (仮)
# Copyright (C) 2007-2008 HORIKAWA Hisashi. All rights reserved.
#     http://www.nslabs.jp/bside/

# This program is provided under the terms of the Common Public License
# ("Agreement"). Any use, reproduction or distribution of this program
# constitutes recipient's acceptance of the Agreement.
# See the Agreement for more details.
# http://www.ibm.com/developerworks/opensource/library/os-beehive/os-cpl.html



class ApplicationController < ActionController::Base
  # Acts as Authenticatedプラグインを使用する。
  include AuthenticatedSystem
  
  helper :all # include all helpers, all the time

  # See ActionController::RequestForgeryProtection for details
  # Uncomment the :secret if you're not using the cookie session store
  protect_from_forgery # :secret => 'c37e08b75d71b74512e029932a391d84'

  def role_keiri_required
    if !current_user.has_role?(:keiri)
      redirect_to '/'
    end
  end
  private :role_keiri_required
end

#################################################################
# 共通ルーチン


# 3桁ごとにカンマを挿入
def format_amount(n, exp)
  n = Float(n) / (10.0 ** exp) if exp != 0
  intpart, exppart = n.to_s.split('.', 2)
  raise "exp is #{exp}, but n = #{n}" if exppart && exppart.length > exp
  intstr = intpart.reverse.split(/([0-9]{1,3})/).collect {|d| d == "" ? ',' : d}.join('').reverse.chop
  if exp != 0
    sprintf("%s.%s%s", intstr, exppart, "0" * (exp - exppart.length))
  else
    sprintf("%s", intstr)
  end
end

def edit_amount(n, exp)
  if n
    if exp != 0
      (Float(n) / (10.0 ** exp)).to_s
    else
      n.to_s
    end
  else
    ""
  end
end


# 時刻を整形
# ISO 8601だと 2008-01-13T23:51+09 だが見栄えがしない
# TODO:タイムゾーン
def format_time t
  t ? t.strftime("%Y-%m-%d %H:%M") : nil
end

# 年が同じなら短く表示
def format_date_short d, today
  if d
    d.year == today.year ? "#{d.mon}/#{d.day}" : d.to_s
  else
    nil
  end
end

def date_select2 obj, method
  r = date_select obj, method
  r << %Q!<input type="button" onClick="openCalendar(this.form.#{obj}_#{method}_1i, this.form.#{obj}_#{method}_2i, this.form.#{obj}_#{method}_3i, event, &#39;yy/mm/dd&#39;);" value="..." style="padding:0">!
  return r
end

def date_from_select(s, name)
  return Date.new(s["#{name}(1i)"].to_i, s["#{name}(2i)"].to_i, s["#{name}(3i)"].to_i)
end

################ 文字コード関係

# UTF-8 to Shift_JIS (cp932)
def utf2sjis s
  if s
    NKF.nkf('-m0 -s -W', s)
  else
    nil
  end
end

def sjis2utf s
  if s
    NKF.nkf('-w -S -m0', s)
  else
    nil
  end
end

# Unicodeの正規化 (デフォルト = NFKC)
def normalize s, form = :kc
  s ? ActiveSupport::Multibyte::Handlers::UTF8Handler.normalize(s, form) : nil
end
alias :normalize_kc :normalize
