
# -*- encoding:utf-8 -*-
# 経理システム (仮)
# Copyright (C) 2007-2008 HORIKAWA Hisashi. All rights reserved.
#     http://www.nslabs.jp/bside/

# This program is provided under the terms of the Common Public License
# ("Agreement"). Any use, reproduction or distribution of this program
# constitutes recipient's acceptance of the Agreement.
# See the Agreement for more details.
# http://www.ibm.com/developerworks/opensource/library/os-beehive/os-cpl.html



# 添付ファイル
class AttachFilesController < ApplicationController
  before_filter :login_required
  
  # GETs should be safe (see http://www.w3.org/2001/tag/doc/whenToUseGet.html)
  verify :method => :post, :only => [ :destroy, :create, :update ],
         :redirect_to => { :action => :list }
  
  # （自部門の）一覧表示
  def list
    div = current_user.default_division
    raise if !div
    @cdiv = div.default_controller
    raise if !@cdiv

    @attach_files = AttachFile.find :all,
        :select => 'id, original_filename, content_type, attach_kind_id, remarks, division_id, created_by, created_at',
        :conditions => ['division_id IN (SELECT division_id FROM controllers_divisions WHERE control_division_id = ?)', @cdiv]
  end

  def show
    @attach_file = AttachFile.find(params[:id])
  end
  
  # 新規アップロード画面
  def new
    @attach_file = AttachFile.new
    @partner = Partner.find(params[:partner]) if params[:partner]
    @invoice = Invoice.find(params[:invoice]) if params[:invoice]
  end
  
  # ファイルをアップロードする
  def upload
    up = params[:file]
    if up[:partner]
      @partner = Partner.find(up[:partner])
      to_opt = {:controller => 'partners', :action => 'show', :id => @partner}
    elsif up[:invoice]
      @invoice = Invoice.find(up[:invoice])
      to_opt = {:controller => 'invoices', :action => 'show', :id => @invoice}
    else
      to_opt = {:action => 'list'}
    end
    
    AttachFile.transaction {
      file = AttachFile.new(
                  :attach_kind_id => up[:kind],
                  :remarks => up[:remarks],
                  :created_by => current_user,
                  :division => current_user.default_division,
                  :data => up[:file].read,
                  :original_filename => up[:file].original_filename,
                  :content_type => up[:file].content_type)
      file.save!
      ref = AttachFileRef.new(
                  :attach_file => file,
                  :partner => @partner,
                  :invoice => @invoice)
      ref.save!
    }
    flash[:notice] = "ファイルをアップロードしました。"
    redirect_to to_opt
    #else
    #render :action => 'new', :id => @partner || @invoice
    #end
  end

  def edit
    @attach_file = AttachFile.find(params[:id])
  end

  def update
    @attach_file = AttachFile.find(params[:id])
    if @attach_file.update_attributes(params[:attach_file])
      flash[:notice] = 'AttachFile was successfully updated.'
      redirect_to :action => 'show', :id => @attach_file
    else
      render :action => 'edit'
    end
  end

  # ファイル内容を出力する
  def download
    file = AttachFile.find params[:id]
    send_data file.data, :filename => file.original_filename, :type => file.content_type
  end
  
  # エントリを削除する
  def destroy
    AttachFile.transaction {
      file = AttachFile.find(params[:id])
      AttachFileRef.delete_all ['attach_file_id = ?', file.id]
      file.destroy
    }
    flash[:notice] = "添付ファイルを削除しました。"
    redirect_to :action => 'list'
  end

end
