
# -*- encoding:utf-8 -*-
# 経理システム (仮)
# Copyright (C) 2007-2008 HORIKAWA Hisashi. All rights reserved.
#     http://www.nslabs.jp/bside/

# This program is provided under the terms of the Common Public License
# ("Agreement"). Any use, reproduction or distribution of this program
# constitutes recipient's acceptance of the Agreement.
# See the Agreement for more details.
# http://www.ibm.com/developerworks/opensource/library/os-beehive/os-cpl.html




require 'validating_base'

class SearchBox < ValidatingBase
  attr_accessor :bank_name, :branch_name
  
  def validate
    if bank_name.to_s.strip == '' && branch_name.to_s.strip == ''
      errors.add_to_base '銀行名または支店名を入力してください。'
    end
    bank_name = bank_name.to_s.strip
    branch_name = branch_name.to_s.strip
  end
end

# 日本国内の銀行マスタ
# 銀行と支店の組
class BanksController < ApplicationController
  before_filter :login_required
  
  # GETs should be safe (see http://www.w3.org/2001/tag/doc/whenToUseGet.html)
  verify :method => :post, :only => [ :destroy, :create, :update ],
         :redirect_to => { :action => :list }
  
  # 検索画面
  def index
  end
  
  def search
    @search = SearchBox.new params[:search]
    @search.validate
    if !@search.errors.empty?
      render :action => 'index'
      return 
    end
    
    text_b = '%' + Bank.zen2han(@search.bank_name) + '%'
    b_zen = '%' + normalize_kc(@search.bank_name) + '%'
    text_br = '%' + Bank.zen2han(@search.branch_name) + '%'
    br_zen = '%' + normalize_kc(@search.branch_name) + '%'

    if @search.branch_name == ''
      # 銀行だけ
      @s = Bank.find :all,
               :conditions => ['name LIKE ? OR kana_name LIKE ?', b_zen, text_b],
               :limit => 31
    else
      if @search.bank_name == ''
        # 支店だけ
        @s = BankBranch.find :all,
               :conditions => ['name LIKE ? OR kana_name LIKE ?', br_zen, text_br],
               :limit => 31
      else
        # 両方
        @s = BankBranch.find :all,
             :select => 'bank_branches.*', 
             :joins => 'JOIN banks ON bank_branches.bank_id = banks.id',
             :conditions => ['(bank_branches.name LIKE ? OR bank_branches.kana_name LIKE ?) AND (banks.name LIKE ? OR banks.kana_name LIKE ?)', br_zen, text_br, b_zen, text_b],
             :limit => 31
      end
    end
  end
  
  # 銀行の一覧
  def list
    @banks = Bank.paginate :page => params[:page], :per_page => 30
  end
  
  # 銀行の詳細
  def show
    @bank = Bank.find params[:id]
  end

  def new
    @bank = Bank.new
  end

  def create
    @bank = Bank.new(params[:bank])
    @bank.id = params[:bank_code]
    if @bank.save
      flash[:notice] = 'Bank was successfully created.'
      redirect_to :action => 'list'
    else
      render :action => 'new'
    end
  end

  def edit
    @bank = Bank.find(params[:id])
  end

  def update
    @bank = Bank.find(params[:id])
    if @bank.update_attributes(params[:bank])
      flash[:notice] = 'Bank was successfully updated.'
      redirect_to :action => 'show', :id => @bank
    else
      render :action => 'edit'
    end
  end

  def destroy
    Bank.find(params[:id]).destroy
    redirect_to :action => 'list'
  end
end
