
# -*- encoding:utf-8 -*-
# 経理システム (仮)
# Copyright (C) 2007-2008 HORIKAWA Hisashi. All rights reserved.
#     http://www.nslabs.jp/bside/

# This program is provided under the terms of the Common Public License
# ("Agreement"). Any use, reproduction or distribution of this program
# constitutes recipient's acceptance of the Agreement.
# See the Agreement for more details.
# http://www.ibm.com/developerworks/opensource/library/os-beehive/os-cpl.html




# コストセンタ
# TODO: クラス名の変更
class ControlDivisionsController < ApplicationController
  # GETs should be safe (see http://www.w3.org/2001/tag/doc/whenToUseGet.html)
  verify :method => :post, :only => [ :destroy, :create, :update ],
         :redirect_to => { :action => 'index' }

  # 一覧表示
  def index
    if params[:root]
      @root_controller = ControlDivision.find params[:root]
    else
      @root_controller = ControlDivision.roots.first 
    end
    if !@root_controller
      redirect_to :action => 'new_root'
      return
    end

    @control_divisions = ControlDivision.paginate :page => params[:page],
                                                  :per_page => 20,
                          :conditions => ['parent_id = ?', @root_controller.id]

    # コストセンタに結びついていない部門
    @dangling_divisions = Division.find_by_sql [<<EOF, @root_controller.id]
SELECT divisions.*
    FROM divisions LEFT JOIN (
        SELECT control_divisions.id,
               controllers_divisions.division_id
            FROM control_divisions JOIN controllers_divisions
                ON control_divisions.id = controllers_divisions.control_division_id
            WHERE parent_id = ?) s
        ON divisions.id = s.division_id
    WHERE s.id IS NULL
EOF
  end

  def show
    @control_division = ControlDivision.find(params[:id])
  end

  def new
    @control_division = ControlDivision.new
    @root = ControlDivision.find params[:root]
  end
  
  # 作成する
  def create
    @control_division = ControlDivision.new params[:control_division]
    ControlDivision.transaction {
      root = ControlDivision.find params[:root]
      if @control_division.save
        @control_division.move_to_child_of root

        flash[:notice] = 'ControlDivision was successfully created.'
        redirect_to :action => 'index', :root => root
      else
        render :action => 'new'
      end
    }
  end

  def edit
    @control_division = ControlDivision.find(params[:id])
    @root = @control_division.root
  end

  # 更新。ルートの付け替えはできない。
  def update
    @control_division = ControlDivision.find params[:id]
    @control_division.attributes = params[:control_division]
    if @control_division.save
      flash[:notice] = 'ControlDivision was successfully updated.'
      redirect_to :action => 'show', :id => @control_division
    else
      render :action => 'edit'
    end
  end

  def destroy
    ControlDivision.find(params[:id]).destroy
    redirect_to :action => 'index'
  end
  
  # 新しい集計木を作る（画面）
  def new_root
    @root = ControlDivision.new
  end

  # 集計木の編集画面
  def edit_root
  end

  # 部門集計木の生成
  def create_root
    @root = ControlDivision.new params[:root]
    if @root.save
      flash[:notice] = '新しい部門集計木を作成しました。'
      redirect_to :action => 'index', :root => @root
    else
      render :action => 'new_root'
    end
  end
end

