
# -*- encoding:utf-8 -*-
# 経理システム (仮)
# Copyright (C) 2007-2008 HORIKAWA Hisashi. All rights reserved.
#     http://www.nslabs.jp/bside/

# This program is provided under the terms of the Common Public License
# ("Agreement"). Any use, reproduction or distribution of this program
# constitutes recipient's acceptance of the Agreement.
# See the Agreement for more details.
# http://www.ibm.com/developerworks/opensource/library/os-beehive/os-cpl.html




# 会計部門
# 組織コードを含む前提
class DivisionsController < ApplicationController
  before_filter :login_required

  # GETs should be safe (see http://www.w3.org/2001/tag/doc/whenToUseGet.html)
  verify :method => :post, :only => [ :destroy, :create, :update, :update_addable ],
         :redirect_to => { :action => :list }

  def index
    list
    render :action => 'list'
  end

  # 一覧表示
  def list
    @divisions = Division.paginate :page => params[:page], :per_page => 25,
                                   :order => 'division_code'
    @page = params[:page]
  end

  def show
    @division = Division.find(params[:id])
  end

  def new
    @division = Division.new
  end
  
  # 作成
  def create
    @division = Division.new params[:division]
    @division.end_day = nil if !params[:disable]
    if @division.save
      flash[:notice] = 'Division was successfully created.'
      redirect_to :action => 'list'
    else
      render :action => 'new'
    end
  end
  
  # 編集画面
  def edit
    @division = Division.find(params[:id])
    @page = params[:page]
  end

  def update
    @division = Division.find params[:id]
    @division.attributes = params[:division]
    @division.end_day = nil if !params[:disable]
    Division.transaction {
      if @division.save
        ControllersDivision.delete_all ['division_id = ?', @division.id]
        params[:ctr].each {|cid|
          cdiv = ControlDivision.find cid
          ControllersDivision.create :control_division => cdiv, :division => @division
        }
        flash[:notice] = 'Division was successfully updated.'
        redirect_to :action => 'show', :id => @division
      else
        render :action => 'edit'
      end
    }
  end
  
  # 削除する
  def destroy
    div = Division.find params[:id]
    raise if !div
    # コストセンタから切り離す
    ControllersDivision.delete_all ['division_id = ?', div.id]
    div.destroy
    redirect_to :action => 'list'
  end
  
  # 購買伝票で使用可にするかの一覧
  def select_addable
    @divisions = Division.find(:all,
                       :conditions => 'end_day is NULL',
                       :order => 'division_code')
  end
  
  # 更新する
  def update_addable
    list = params[:ss]
    Division.transaction {
      Division.find(:all).each {|r|
        orig_active = r.purchase_active
        r.purchase_active = list.index(r.id.to_s) ? 1 : 0   # activeでチェックを入れる
        r.save! if orig_active != r.purchase_active
      }
    }
    redirect_to :action => 'select_addable'
  end
end
