
# -*- encoding:utf-8 -*-
# 経理システム (仮)
# Copyright (C) 2007-2008 HORIKAWA Hisashi. All rights reserved.
#     http://www.nslabs.jp/bside/

# This program is provided under the terms of the Common Public License
# ("Agreement"). Any use, reproduction or distribution of this program
# constitutes recipient's acceptance of the Agreement.
# See the Agreement for more details.
# http://www.ibm.com/developerworks/opensource/library/os-beehive/os-cpl.html




# 経理だけが使う関係
class KeiriController < ApplicationController
  before_filter :login_required
  before_filter :role_keiri_required
  

  def index
  end
  
  # 外貨建の金額を円に換算
  def get_yen_amount entry_day, amount, currency
    # 直近の日付を探して
    cr = CurrencyRate.find :first,
                           :select => 'MAX(date) as date',
                           :conditions => ['currency_id = ? AND date <= ?',
                                           currency.id, entry_day]
    raise "not found currency rate: currency = #{currency.name}, entry_day = #{entry_day}" if !cr || !cr.date

    # 適切なレコードを取得する
    cr = CurrencyRate.find :first,
                           :conditions => ['currency_id = ? AND date = ?',
                                           currency.id, cr.date]
    return [(Float(amount) * cr.rate / 10 ** currency.exponent + 0.5).floor, cr.rate]
  end
  private :get_yen_amount

  # 仕訳確認画面
  def export
    @company = CompanySetting.find :first
    functional_currency = @company.functional_currency
    raise if !functional_currency
    fixed_day = @company.fixed_gl_day
    
    # TODO: 経理承認したときに、出力待ちテーブルにpushしたほうがいいかなぁ

    @entries = []
    entry_ids = []; si_entry_ids = []
    @deferred = []
    @deferred_si = []

    # 購買請求書
    invoices = Invoice.find :all,
                            :conditions => 'state = 10',
                            :order => 'buy_day'
    invoices.each {|inv|
      entry_day = [inv.buy_day, fixed_day + 1].max
      if inv.currency_id != functional_currency.id &&
                  entry_day > @company.currency_limit
        @deferred << inv.id
      else
        @entries += make_entry(@company, inv, entry_day)
        entry_ids << inv.id
      end
    }

    # 販売請求書
    invoices = SalesInvoice.find :all,
                                 :conditions => 'state = 10',
                                 :order => 'sell_day'
    invoices.each {|inv|
      entry_day = [inv.sell_day, fixed_day + 1].max
      if inv.currency_id != functional_currency.id &&
                          entry_day > @company.currency_limit
        @deferred_si << inv.id
      else
        @entries += make_si_entry(@company, inv, entry_day)
        si_entry_ids << inv.id
      end
    }

    @size = entry_ids.size + si_entry_ids.size
    session[:entries] = entry_ids
    session[:si_entries] = si_entry_ids
  end

  # 実際の出力
  def export2
    company = CompanySetting.find(:first)
    fixed_day = company.fixed_gl_day
    
    @entries = []
    
    # 購買請求書
    inv_ids = session[:entries]
    inv_ids.each {|inv_id|
      inv = Invoice.find inv_id
      entry_day = [inv.buy_day, fixed_day + 1].max
      @entries += make_entry(company, inv, entry_day)
    }
    
    # 販売請求書
    si_ids = session[:si_entries]
    si_ids.each {|inv_id|
      inv = SalesInvoice.find inv_id
      entry_day = [inv.sell_day, fixed_day + 1].max
      @entries += make_si_entry(company, inv, entry_day)
    }

    headers["Content-Type"] = "application/vnd.ms-excel; charset=UTF-8"
    headers["Content-Disposition"] = "attachment;\n  filename*=iso-2022-jp'ja'" +
                                     NKF.nkf("-m0 -j -W", "purchase.csv")
    render :action => 'export2', :layout => false

    # 連動済みとしてマークする。
    Invoice.transaction {
      inv_ids.each {|inv_id|
        inv = Invoice.find inv_id
        inv.state = 20
        inv.journal_day = [inv.buy_day, fixed_day + 1].max
        inv.save!
      }
      
      si_ids.each {|inv_id|
        inv = SalesInvoice.find inv_id
        inv.state = 20
        inv.journal_day = [inv.sell_day, fixed_day + 1].max
        inv.save!
      }
    } # of transaction
  end
  
  # 販売請求書 -> 仕訳
  def make_si_entry company, inv, entry_day
    result = []
    inv.si_journals.each {|line|
      amount, rate = get_yen_amount(entry_day, line.amount, inv.currency)
      e = {
        :entry_day => entry_day,
        :entry_no => 40000 + inv.id,  # TODO:またもや決め打ち。fixme

        :dr_account_title => line.account_title.receivable,
        :dr_division => nil,
        :dr_partner_code => inv.partner.zaimu_id,

        :cr_account_title => line.account_title,
        :cr_division => line.division,
        :cr_partner_code => nil,

        :amount => amount,
        :vat => line.vat_code,
        :due_day => inv.due_day,
        :created_by => inv.created_by
      }
      e[:remarks] = [line.division.name, inv.partner.name, line.remarks].join('/')
      if inv.currency.id != company.functional_currency.id
        # raise "#{inv.currency.code.inspect}, #{rate.inspect}" # DEBUG
        e[:remarks] << " " <<
                       format_amount(line.amount, inv.currency.exponent) <<
                       inv.currency.code << "@" << rate.to_s
      end
      result << e
    }
    return result
  end
  private :make_si_entry

  # 購買請求書 -> 仕訳
  def make_entry company, inv, entry_day
    result = []
    inv.invoice_details.each {|detail|
      amount, rate = get_yen_amount(entry_day, detail.amount, inv.currency)
      line = {
        :entry_day => entry_day,
        :entry_no => 10000 + inv.id,  # TODO:決め打ち。fixme
        
        # 借方
        :dr_account_title => detail.account_title,
        :dr_division => detail.division,
        :dr_partner_code => (detail.account_title.req_partner ? 
                           (inv.bank_account ? inv.bank_account.zaimu_ac :
                                               inv.partner.zaimu_id) : nil),
        
        # 貸方
        :cr_account_title => detail.account_title.payable,
        :cr_division => nil,
        :cr_partner_code => (inv.bank_account ? inv.bank_account.zaimu_ac : inv.partner.zaimu_id),

        :amount => amount,
        :vat => detail.account_title.get_real_vat_code(detail.vat_code), # for compat.
        :due_day => (inv.bank_account ? inv.due_day : nil),
        :created_by => inv.created_by,
      }
      if inv.currency.id == company.functional_currency.id
        line[:remarks] = [detail.division.name, inv.partner.name,
                            detail.remarks].join('/')
      else
        line[:remarks] = [detail.division.name, inv.partner.name,
          "#{detail.remarks} #{format_amount(detail.amount, inv.currency.exponent)}#{inv.currency.code}@#{rate}"].join('/')
      end
      result << line
    }
    return result
  end
  private :make_entry

  # 総勘定元帳 (仕訳) のインポート画面
  def import_gl
    require 'csv'

    @fname = '/shiwake.csv'
    if FileTest.exist?(IMPORT_DATA_DIR + @fname)
      data = sjis2utf File.read(IMPORT_DATA_DIR + @fname)
      @head = []
      
      idx = 0
      CSV::Reader.parse(data) {|line|
        idx += 1; next if idx == 1
        break if idx == 10
        @head << line
      }
    end
  end
  
  SCRIPT_DIR = File.dirname(__FILE__) + '/../../script'
  def do_import_gl
    `#{KeiriController::SCRIPT_DIR}/run_import_gl.rb`
    redirect_to :action => 'import_gl_log'
  end
  
  # ログを表示
  def import_gl_log
    @out = File.read KeiriController::SCRIPT_DIR + '/import_gl_out.log'
    @err = File.read KeiriController::SCRIPT_DIR + '/import_gl_err.log'
  end
  
  # 取引先（支払先）のインポート画面
  def import_partners
    `#{KeiriController::SCRIPT_DIR}/run_import_partners.rb`
    redirect_to :action => 'import_partners_log'
  end
  
  # 勘定科目のインポート
  def import_account_titles
    `#{KeiriController::SCRIPT_DIR}/run_import_account_titles.rb`
    redirect_to :action => 'import_account_titles_log'
  end
  
  # 社員マスタアップロード画面
  def import_workers
  end

  # ログ表示
  def import_partners_log
    @out = File.read KeiriController::SCRIPT_DIR + '/import_partners_out.log'
    @err = File.read KeiriController::SCRIPT_DIR + '/import_partners_err.log'
  end
  
  # 勘定科目のインポートのログ表示
  def import_account_titles_log
    @out = File.read KeiriController::SCRIPT_DIR + '/import_account_titles_out.log'
    @err = File.read KeiriController::SCRIPT_DIR + '/import_account_titles_err.log'
  end

end
