
# -*- encoding:utf-8 -*-
# 経理システム (仮)
# Copyright (C) 2007-2008 HORIKAWA Hisashi. All rights reserved.
#     http://www.nslabs.jp/bside/

# This program is provided under the terms of the Common Public License
# ("Agreement"). Any use, reproduction or distribution of this program
# constitutes recipient's acceptance of the Agreement.
# See the Agreement for more details.
# http://www.ibm.com/developerworks/opensource/library/os-beehive/os-cpl.html




require 'validating_base'

class BankAccountSearchBox < ValidatingBase
  attr_accessor :text
end

# 自社銀行口座
class OurBanksController < ApplicationController

  # GETs should be safe (see http://www.w3.org/2001/tag/doc/whenToUseGet.html)
  verify :method => :post, :only => [ :destroy, :create, :update ],
         :redirect_to => { :action => :list }
  
  def index
    list
    render :action => 'list'
  end

  def list
    #@our_bank_pages, @our_banks = paginate :our_banks, :per_page => 10
    @our_banks = OurBank.paginate :page => params[:page], :per_page => 20
  end

  def show
    @our_bank = OurBank.find(params[:id])
  end
  
  # 新しい口座（の選択）
  def new
    @our_bank = OurBank.new
  end
  
  def new2
    @search = BankAccountSearchBox.new params[:search]
    if @search.text.to_s.strip == ''
      @search.errors.add_to_base '検索テキストを入力してください。'
      render :action => 'new'
      return
    end
    t = '%' + Bank.zen2han(@search.text.strip) + '%' # TODO:fixme
    if @search.text.to_i != 0
      @bank_accounts = BankAccount.find :all, :conditions => ['ac_number = ?', @search.text], :limit => 31
    else
      @bank_accounts = BankAccount.find :all, :conditions => ['kana_name like ?', t], :limit => 31
    end
  end
  
  def new3
    @bank_account = BankAccount.find params[:bank_account]
  end

  # 作成
  def create
    @our_bank = OurBank.new params[:our_bank]
    @our_bank.bank_account_id = params[:ba]
    if @our_bank.save
      flash[:notice] = 'OurBank was successfully created.'
      redirect_to :action => 'list'
    else
      render :action => 'new'
    end
  end
  
  # 編集画面
  def edit
    @our_bank = OurBank.find(params[:id])
    @bank_account = @our_bank.bank_account
  end

  def update
    @our_bank = OurBank.find(params[:id])
    if @our_bank.update_attributes(params[:our_bank])
      flash[:notice] = 'OurBank was successfully updated.'
      redirect_to :action => 'show', :id => @our_bank
    else
      render :action => 'edit'
    end
  end

  def destroy
    OurBank.find(params[:id]).destroy
    redirect_to :action => 'list'
  end
end
