
# -*- encoding:utf-8 -*-
# 経理システム (仮)
# Copyright (C) 2007-2008 HORIKAWA Hisashi. All rights reserved.
#     http://www.nslabs.jp/bside/

# This program is provided under the terms of the Common Public License
# ("Agreement"). Any use, reproduction or distribution of this program
# constitutes recipient's acceptance of the Agreement.
# See the Agreement for more details.
# http://www.ibm.com/developerworks/opensource/library/os-beehive/os-cpl.html





require 'upload_module'
require 'partner_search_box'

# 取引先
class PartnersController < ApplicationController
  include UploadModule
  before_filter :login_required
  
  # GETs should be safe (see http://www.w3.org/2001/tag/doc/whenToUseGet.html)
  verify :method => :post, :only => [ :destroy, :create, :update ],
         :redirect_to => { :action => :list }

  # 検索フォームの表示
  def index
  end
  
  def search
    @search = PartnerSearchBox.new(params[:search])
    if !@search.text || @search.text.strip == ''
      @search.errors.add_to_base '検索テキストを入力してください。'
      render :action => 'index'
      return
    end

    @partners = @search.find_by_text
  end

  def show
    @partner = Partner.find params[:id]
    # 請求書リスト
    @invoices = Invoice.paginate :page => params[:page], :per_page => 25,
                             :conditions => ['partner_id = ?', @partner.id],
                             :order => 'buy_day DESC'
  end

  def new
    @partner = Partner.new
  end
  
  # 10刻みで加算する
  def get_next_zaimu_id
    cs = CompanySetting.find(:first)
    raise if !cs
    while Partner.find_by_zaimu_id(cs.partner_counter)
      cs.partner_counter += 10 - (cs.partner_counter % 10)
    end
    r = cs.partner_counter
    cs.partner_counter += 10
    cs.save!
    return r
  end
  private :get_next_zaimu_id
  
  # 新しく作成
  def create
    @partner = Partner.new params[:partner]
    @partner.zaimu_id = get_next_zaimu_id() if @partner.zaimu_id.to_i == 0
    @partner.updated_by = current_user
    if @partner.save
      flash[:notice] = '取引先は、正常に作成されました。'
      redirect_to :action => 'show', :id => @partner
    else
      render :action => 'new'
    end
  end
  
  # 編集画面の表示
  def edit
    @partner = Partner.find(params[:id])
  end
  
  # 更新
  def update
    @partner = Partner.find(params[:id])
    @partner.updated_by = current_user
    if @partner.update_attributes(params[:partner])
      flash[:notice] = 'Partner was successfully updated.'
      redirect_to :action => 'show', :id => @partner
    else
      render :action => 'edit'
    end
  end

  def destroy
    partner = Partner.find(params[:id])
    s =  "取引先 #{partner.name} は、正常に削除されました。"
    partner.destroy
    flash[:notice] = s
    redirect_to :action => 'index'
  end
end
