
# -*- encoding:utf-8 -*-
# 経理システム (仮)
# Copyright (C) 2007-2008 HORIKAWA Hisashi. All rights reserved.
#     http://www.nslabs.jp/bside/

# This program is provided under the terms of the Common Public License
# ("Agreement"). Any use, reproduction or distribution of this program
# constitutes recipient's acceptance of the Agreement.
# See the Agreement for more details.
# http://www.ibm.com/developerworks/opensource/library/os-beehive/os-cpl.html


require 'tempfile'


# 売上側の請求書 (売掛金管理)
class SalesInvoicesController < ApplicationController
  before_filter :login_required
  append_after_filter :make_pdf, :only => [:print_invoice, :print_invoice_en]

  # GETs should be safe (see http://www.w3.org/2001/tag/doc/whenToUseGet.html)
  verify :method => :post, :only => [ :destroy, :create, :update ],
         :redirect_to => { :action => :list }

  def index
    list
    render :action => 'list'
  end
  

  # XSL-FOをPDFに変換する
  def make_pdf
    inf = Tempfile.new('body', RAILS_ROOT + "/tmp")
    inf.write(response.body)
    inf.close

    (outf = Tempfile.new('out', RAILS_ROOT + "/tmp")).close
    (errf = Tempfile.new('err', RAILS_ROOT + "/tmp")).close

    headers['Content-Type'] = 'application/pdf'
    `#{ENV['FOP_HOME']}/fop -c #{RAILS_ROOT}/config/fop-conf.xml #{inf.path} -pdf #{outf.path} 2>#{errf.path}`
    response.body = File.read(outf.path)

    inf.unlink
    outf.unlink
  end
  private :make_pdf
  
  # 作った請求書を出力
  def print_invoice
    @company = CompanySetting.find(:first)
    @contact = OurContact.find :first, :conditions => "lang = 'ja'"
    @invoice = SalesInvoice.find params[:id]
    if !@invoice.si_number
      @invoice.si_number = (@company.si_counter += 1)
    end

    File.open(RAILS_ROOT + "/tmp/logo.png", "w") {|fp|
      fp.write @company.logo_image
    }

    SalesInvoice.transaction {
      @company.save!
      @invoice.save!
    }

    render :layout => false
  end
  
  def print_invoice_en
    @contact = OurContact.find :first, :conditions => "lang = 'en'"
    @company = CompanySetting.find(:first)
    @invoice = SalesInvoice.find params[:id]
    render :layout => false
  end
  
  def list
    @sales_invoices = SalesInvoice.paginate :page => params[:page],
                                            :per_page => 20
  end

  def show
    @invoice = SalesInvoice.find(params[:id])
  end
  
  # 新規作成画面
  def new
    @partner = Partner.find params[:partner]
    @invoice = SalesInvoice.new
    @invoice.contact_name1 = @partner.name
  end
  
  # 明細行のレコードを作る
  def create_details new_exp
    result = []
    (0..9).each {|i| # magic
      pa = params["detail#{i}"]
      detail = SiDetail.new pa
      detail.attributes = {
        :sales_invoice => @invoice,
        :unit_price => (pa[:unit_price].blank? ? nil :
                          (pa[:unit_price].to_f * (10 ** new_exp) + 0.5).floor),
        :line_no => i + 1
      }
      if detail.unit_price.to_i != 0 && detail.qty.to_i != 0
        detail.amount = detail.unit_price * detail.qty
      else
        detail.amount = (pa[:amount].to_f * (10 ** new_exp) + 0.5).floor
      end
      result << detail
    }
    # 末尾の空白行を取り除く
    while last = result.last
      if last.amount != 0 || last.description.strip != ''
        break
      else
        result.pop
      end
    end
    return result
  end
  private :create_details
  
  # 仕訳レコードを生成
  def create_journals attrs, details
    result = []
    # まず、何行の仕訳を生成するか
    v = {10 => 0, 30 => 0, 40 => 0}
    details.each {|line|
      next if line.amount == 0 # amountは整数に正規化されている
      if line.vat_code == 20
        v[10] += line.amount
      else
        v[line.vat_code] += line.amount
      end
    }
    v[10] += @invoice.ex_vat_amount  # 外税の消費税

    v.each {|vat_code, amount|
      next if amount == 0
      j = SiJournal.new attrs
      j.remarks = details.first.description if j.remarks.to_s.strip == ''
      j.sales_invoice = @invoice
      j.vat_code = vat_code
      j.amount = amount
      result << j
    }
    return result
  end
  private :create_journals

  # 作成
  def create
    @partner = Partner.find params[:partner]
    @invoice = SalesInvoice.new params[:invoice]
    @invoice.attributes = {
      :partner => @partner,
      :division => current_user.default_division,
      :created_by => current_user,
      :state => 0,
    }
    details = create_details(@invoice.currency.exponent)
    banks = params[:ba].blank? ? [] : OurBank.find(params[:ba])

    SalesInvoice.transaction {
      details.each {|r|
        @invoice.si_details << r
      }

      if @invoice.save
        details.each {|r| r.save! }
        banks.each {|r|
          b = SiBank.new :sales_invoice => @invoice,
                         :our_bank => r
          b.save!
        }
        # 仕訳を作る
        journals = create_journals params[:journal], details
        journals.each {|j| j.save!}

        flash[:notice] = 'SalesInvoice was successfully created.'
        redirect_to :action => 'show', :id => @invoice
      else
        render :action => 'new', :partner => @partner
      end
    } # of transaction
  end
  
  # 編集画面
  def edit
    @invoice = SalesInvoice.find(params[:id])
    @partner = @invoice.partner
  end
  
  # 更新する
  def update
    orig_invoice = SalesInvoice.find params[:id]
    @partner = orig_invoice.partner
    @invoice = SalesInvoice.new params[:invoice] # 検証用のダミー
    details = create_details(@invoice.currency.exponent)
    details.each {|r| @invoice.si_details << r }
    
    if !@invoice.valid?
      @invoice.id = orig_invoice.id
      render :action => 'edit', :id => @invoice
      return
    end
    
    # 更新する
    SalesInvoice.transaction {
      @invoice = orig_invoice
      @invoice.attributes = params[:invoice]
      @invoice.save!

      # 詳細行は作り直す
      SiDetail.delete_all ['sales_invoice_id = ?', @invoice.id]
      details.each {|r|
        r.sales_invoice = @invoice
        r.save!
      }

      # 振込先口座も作り直す
      SiBank.delete_all ['sales_invoice_id = ?', @invoice.id]
      if params[:ba]
        banks = OurBank.find params[:ba]
        banks.each {|r|
          b = SiBank.new :sales_invoice => @invoice,
                       :our_bank => r
          b.save!
        }
      end
      
      SiJournal.delete_all ['sales_invoice_id = ?', @invoice.id]
      journals = create_journals params[:journal], details
      journals.each {|j| j.save!}
      
      il = SiLog.new :sales_invoice => @invoice,
                     :kind => SiLog::MODIFY,
                     :remarks => '',
                     :done_by => current_user,
                     :done_at => Time.now
      il.save!
    } # of transaction

    flash[:notice] = '請求書は正常に更新されました。'
    redirect_to :action => 'show', :id => @invoice
  end

  # 請求書発行依頼の表示
  def print_request
    @invoice = SalesInvoice.find params[:id]
  end
  
  # 削除
  def destroy
    inv = SalesInvoice.find params[:id]
    partner = inv.partner
    SalesInvoice.transaction {
      cond = ['sales_invoice_id = ?', inv.id]
      SiDetail.delete_all cond
      SiBank.delete_all cond
      SiJournal.delete_all cond
      SiLog.delete_all cond
      inv.destroy
    }
    flash[:notice] = '請求書を削除しました。'
    redirect_to :controller => 'partners', :action => 'show', :id => partner
  end
  
  # 承認
  def sign_off
    @invoice = SalesInvoice.find params[:id]
    SalesInvoice.transaction {
      if @invoice.state < SalesInvoice::SIGNED_OFF
        il = SiLog.new(:sales_invoice => @invoice,
                       :kind => SiLog::SIGN_OFF,
                       :remarks => "",
                       :done_by => current_user,
                       :done_at => Time.now)
        il.save!
        @invoice.state = SalesInvoice::SIGNED_OFF
        @invoice.save!
      end
    }
    flash[:notice] = '請求書を承認しました。'
    redirect_to :action => 'show', :id => @invoice
  end
  
  # コピーして新しい請求書を起票
  def copy_new
    @invoice = SalesInvoice.find params[:id]
    @partner = @invoice.partner
    @invoice.sell_day = ((@invoice.sell_day + 1) >> 1) - 1
    @invoice.due_day = ((@invoice.due_day + 1) >> 1) - 1
    render :action => 'new'
  end
end
