
# -*- encoding:utf-8 -*-
# 経理システム (仮)
# Copyright (C) 2007-2008 HORIKAWA Hisashi. All rights reserved.
#     http://www.nslabs.jp/bside/

# This program is provided under the terms of the Common Public License
# ("Agreement"). Any use, reproduction or distribution of this program
# constitutes recipient's acceptance of the Agreement.
# See the Agreement for more details.
# http://www.ibm.com/developerworks/opensource/library/os-beehive/os-cpl.html




require 'validating_base'

class SearchBox < ValidatingBase
  attr_accessor :text, :code
end

# ユーザ（社員）
class UsersController < ApplicationController
  before_filter :login_required
  
  # GETs should be safe (see http://www.w3.org/2001/tag/doc/whenToUseGet.html)
  verify :method => :post, :only => [ :destroy, :create, :update ],
         :redirect_to => { :action => :list }
  
  # 検索画面
  def index
  end
  
  # 検索結果
  def search
    @search = SearchBox.new(params[:search])
    if @search.text.to_s.strip == '' && @search.code.to_s.strip == ''
      @search.errors.add_to_base '検索テキストを指定してください。'
      render :action => 'index'
      return
    end

    c = []; v = []
    if @search.text.to_s.strip != ''
      text = '%' + normalize_kc(@search.text.strip).upcase + '%' # PostgreSQL only. TODO:fixme
      c << '(upper(name) LIKE ? OR kana_name LIKE ? OR upper(email) LIKE ?)'
      v << text << text << text
    end
    if @search.code.to_s.strip != ''
      c << 'worker_code = ?'
      v << @search.code
    end
    @users = User.find :all,
                       :conditions => [c.join(' AND '), *v],
                       :limit => 40
  end
  
  # 一覧
  def list
    @workers = User.paginate :page => params[:page], :per_page => 25,
                             :order => 'worker_code'
  end

  def show
    @worker = User.find params[:id]
    @exist_gl = GeneralLedger.find :first,
                                   :conditions => ['created_by = ?', @worker.id]
  end

  def new
    @worker = User.new
  end

  def create
    @worker = User.new(params[:worker])
    @worker.active = 1
    if @worker.save
      flash[:notice] = 'Worker was successfully created.'
      redirect_to :action => 'show', :id => @worker
    else
      render :action => 'new'
    end
  end

  def edit
    @worker = User.find params[:id]
  end

  def update
    @worker = User.find(params[:id])
    if @worker.update_attributes(params[:worker])
      flash[:notice] = 'Worker was successfully updated.'
      if current_user.has_role?(:keiri)
        redirect_to :action => 'show', :id => @worker
      else
        redirect_to '/'
      end
    else
      render :action => 'edit'
    end
  end
  
  # 削除
  def destroy
    u = User.find params[:id]
    User.transaction {
      UsersDivision.delete_all ['user_id = ?', u.id]
      u.destroy
    }
    flash[:notice] = "ユーザ #{u.worker_code} #{u.name} を削除しました。"
    redirect_to :action => 'index'
  end
  
  # 所属部門の編集画面
  def edit_divisions
    @worker = User.find params[:id]
    @divisions = @worker.divisions
  end
  
  # 所属部門を更新
  def update_divisions
    require 'set'
    @worker = User.find params[:id]
    new_divs = params[:div].collect {|v| v.to_i}.delete_if {|v| v == 0}
    UsersDivision.transaction {
      # 一度全部消して、新たに作成する
      UsersDivision.delete_all ["user_id = ?", @worker.id]
      new_divs.each_with_index {|d, i|
        r = UsersDivision.new :user => @worker,
                              :division_id => d,
                              :default_pair => (i == 0 ? 1 : 0)
        r.save!
      }
    } # of transaction
    flash[:notice] = "Worker was successfully updated."
    redirect_to :action => 'show', :id => @worker
  end
end
