
# 勘定科目
class AccountTitle < ActiveRecord::Base
  belongs_to :receivable, :foreign_key => 'receivable_id',
                          :class_name => 'AccountTitle'
  belongs_to :payable, :foreign_key => 'payable_id',
                       :class_name => 'AccountTitle'
  
  # 集計先
  has_many :account_title_trees, :foreign_key => 'parent_id'
  #has_many :parents, :through => :account_title_trees
  has_many :children, :through => :account_title_trees

  VAT_NAMES = {10 => '税込5%', 20 => '外税5%', 30 => '非課税', 40 => '―'}

  # 購買請求書で追加可能な科目一覧を得る
  def self.addable()
    find :all,
         :conditions => '(end_day IS NULL) AND (payable_id IS NOT NULL) AND (aggregate_flag IS FALSE)',
         :order => 'account_code, suppl_code'
  end

  # 販売請求書での追加可能な科目一覧
  def self.sales_addable()
    find :all,
         :conditions => '(end_day IS NULL) AND (receivable_id IS NOT NULL) AND (aggregate_flag IS FALSE)',
         :order => 'account_code, suppl_code'
  end

  # TODO: 決め打ち・fixme
  def self.receivable_candidates
    AccountTitle.find :all,
               :conditions => '(end_day IS NULL) AND account_code IN (115, 168)',
               :order => 'account_code, suppl_code' 
  end

  # TODO: 決め打ち・fixme
  def self.payable_candidates
    AccountTitle.find :all,
               :conditions => '(end_day IS NULL) AND account_code IN (305, 321)',
               :order => 'account_code, suppl_code'
  end
  
  validates_presence_of :account_code
  validates_presence_of :suppl_code
  # @override
  def validate
    if sign != 1 && sign != -1
      errors.add 'sign', '内部エラー: 符号は+1または-1に限る'
    end
  end

  def name2
    return "#{account_code} #{name}"
  end
  
  # 自分自身と祖先は禁止
  def self_and_ancestors
    ret = last = [self.id]
    while !last.empty?
      last = AccountTitleTree.find(:all,
                                   :conditions => ['child_id IN (?)', last]
             ).collect {|r| r.parent_id}
      ret += last
    end
    return ret
  end

  # 直接の子供と子孫が被ったら警告
  # 直接の子供ではない子孫を返す
  def descendant
    ret = []
    last = AccountTitleTree.find(:all,
               :conditions => ['parent_id = ?', self.id]).collect {|r| r.child_id}
    while !last.empty?
      last = AccountTitleTree.find(:all,
               :conditions => ['parent_id IN (?)', last]).collect {|r| r.child_id}
      ret += last
    end
    return ret
  end

  # 仕訳での消費税コード
  def get_real_vat_code(vat_or_zero)
    if vat_or_zero.to_i != 0
      vat_or_zero
    else
      if vat_code != 0
        vat_code
      else
        10 # TODO:決め打ち
      end
    end
  end

  def self.vat_ary
    a = VAT_NAMES.to_a.sort {|x, y| x[0] <=> y[0]}
    a.collect {|k, v| [v, k]}
  end
end
