
require 'lib/multiparam_errors'

class Invoice < ActiveRecord::Base
  belongs_to :partner
  belongs_to :division
  belongs_to :bank_account
  belongs_to :currency
  belongs_to :created_by, :foreign_key => 'created_by', :class_name => 'User'
  has_many :invoice_details
  has_many :attach_file_refs
  has_many :attach_files, :through => :attach_file_refs
  has_many :imports_invoices
  has_many :import_declarations, :through => :imports_invoices

  SIGNED_OFF = 10
  
  before_save :normalize_string
  
  def normalize_string
    self.invoice_no = normalize_kc invoice_no
  end

  def sum_amount
    r = InvoiceDetail.find :first,
                           :select => 'sum(amount) AS sum_amount',
                           :conditions => ['invoice_id = ?', id]
    return r.sum_amount.to_i
  end

  # @override
  def validate  # save() -> valid?() -> validate()
    # errors.add('due_day', '支払日が過去になっています。') if due_day < Date.today
    errors.add_to_base('明細が1行もありません。') if invoice_details.empty?
    if due_day.is_a?(Date) && buy_day.is_a?(Date) && (due_day < buy_day)
      errors.add('buy_day', '請求締め日は支払日より前にしてください')
    end
  end
end
