# -*- encoding:utf-8 -*-

# 販売請求書
class SalesInvoice < ActiveRecord::Base
  has_many :si_details
  has_many :si_journals
  has_many :si_banks
  has_many :our_banks, :through => :si_banks
  belongs_to :partner
  belongs_to :division   # 所管部門
  belongs_to :currency
  belongs_to :created_by, :foreign_key => 'created_by', :class_name => 'User'
  
  DELIVERY_TYPE_NAMES = [nil, '担当者持参', '経理から郵送']
  SIGNED_OFF = 10
  
  validates_presence_of :delivery
  # @override
  def validate
    errors.add_to_base('明細が1行もありません。') if si_details.empty?
  end
  
  # 戻り値は (10 ** currency.exponent) した額
  def ex_vat_amount
    r = SiDetail.find :first,
                      :select => 'sum(amount) as ex_total',
                      :conditions => ['sales_invoice_id = ? and vat_code = 20',
                                      self.id]
    ex_total = r && r.ex_total ? r.ex_total.to_i : 0
    return (ex_total * 0.05).floor
  end
end
