
# date_select() で作った日付選択コントロールで不正な日付を入れた場合のエラーを扱う
# http://www.fdiary.net/ml/rails/msg/739

if !$MULTIPARAM__
$MULTIPARAM__ = true

module ActiveRecord
  # @override   base.rb
  # fix handling message.
  class AttributeAssignmentError < ActiveRecordError #:nodoc:
    def initialize message, exception, attribute
      super message
      @exception = exception
      @attribute = attribute
    end
  end

  class Base
    # attributes=() を差し替える、複数引数を検査するメソッド
    def set_attributes_with_multiparam_validation(attributes)
      begin
        set_attributes_without_multiparam_validation(attributes)
      rescue MultiparameterAssignmentErrors => e
        multiparam_errors.concat e.errors
      end
    end
    alias_method :set_attributes_without_multiparam_validation, :attributes=
    alias_method :attributes=, :set_attributes_with_multiparam_validation
    
    # valid?() を差し替える   validations.rb
    def valid_p_with_multiparam_validation
      ret = (valid_p_without_multiparam_validation &&
            multiparam_errors.empty?)
      add_multiparam_errors
      ret
    end
    alias_method :valid_p_without_multiparam_validation, :valid?
    alias_method :valid?, :valid_p_with_multiparam_validation

    private
    HOLDER_CLASSES = {
      Time => Struct.new("TimeHolder",
                             :year, :month, :day, :hour, :min, :sec),
      Date => Struct.new("DateHolder", :year, :month, :day)
    }

    def multiparam_errors
      @multiparam_errors ||= []
    end

    def add_multiparam_errors
      multiparam_errors.each {|error|
        errors.add error.attribute, error.message
      }
      multiparam_errors.clear
    end

    # @override    base.rb
    def execute_callstack_for_multiparameter_attributes(callstack)
      errors = []
      callstack.each do |name, values|
        klass = (self.class.reflect_on_aggregation(name.to_sym) ||
                                             column_for_attribute(name)).klass
        if values.empty?
          send(name + "=", nil)
        else
          begin
            send(name + "=", Time == klass ? (@@default_timezone == :utc ? klass.utc(*values) : klass.local(*values)) : klass.new(*values))
          rescue => ex
            send(name + "=", HOLDER_CLASSES[klass].new(*values))  # この行を追加
            errors << AttributeAssignmentError.new("error on assignment #{values.inspect} to #{name}", ex, name)
          end
        end
      end
      unless errors.empty?
        raise MultiparameterAssignmentErrors.new(errors), "#{errors.size} error(s) on assignment of multiparameter attributes"
      end
    end

  end # of class Base
end

end
