#!/usr/bin/ruby

# 財務大将から取引先をインポートする。
# 銀行口座の登録はすべて財務大将側で行い、
#     見える購買システムでは、親コードへの紐付け、個人旅客の不可視だけおこなう。

# 取引先の名称、コメントなどは購買システムで更新する。インポート時に上書きしない。

# 銀行口座のある相手先は、
#   bank_accounts にレコードがあれば更新
#   なければpartnerと二つレコードを作る
# 銀行口座のない相手先は、
#   partnerにレコードがあれば更新、なければ作る


require File.dirname(__FILE__) + '/import_common.rb'
require 'csv'


# 取引先を探す。
# @return
#   見つからないときはnil
def find_partner zaimu_id
  partner = Partner.find_by_zaimu_id(zaimu_id)
  if !partner && (zaimu_id % 10) != 0
    # 末尾0で登録がないか探してみる
    partner = Partner.find_by_zaimu_id((zaimu_id / 10) * 10)
  end
  return partner
end

# 取引先レコードを探す・作る
def create_partner line
  zaimu_id = line[0].to_i
  if !(partner = find_partner(zaimu_id))
    partner = Partner.new(:name => norm(line[1].split(/[\:：]/u, 2)[0]),
                          :kana_name => norm(line[2] || ""),
                          :zaimu_id => zaimu_id,
                          :remarks => norm(line[1].split(/[\:：]/u, 2)[1] || ""))
    partner.save!
  end
  return partner
end

# 銀行を探す。
# line に銀行データが含まれないときはnil, 銀行データが含まれて、銀行マスタにないときはエラー
def find_bank line
  bank_code = line[29].to_i
  branch_code = line[31].to_i
  return nil if bank_code == 0 || branch_code == 0
  return nil if line[34].to_i == 0 # 口座番号
  return nil if line[35].to_s.strip == ''

  branch = BankBranch.find_by_code(bank_code, branch_code)
  raise "bank not found: (#{bank_code}, #{branch_code})" if !branch
  return branch
end


# 取引先の取り込み
def import_partners
  data = sjis2utf8_x(File.read(DATA_DIR + '/partners.csv'))
  
  Partner.transaction {
    idx = 0
    CSV::Reader.parse(data) {|line|
      idx += 1; next if idx == 1
      zaimu_id = line[0].to_i
      next if zaimu_id == 0
      next if zaimu_id >= 120000 && zaimu_id <= 129999 # 個人旅客
      next if zaimu_id >= 170000 && zaimu_id <= 179999 # 個人旅客2

      up = false
      bank_ac = BankAccount.find_by_zaimu_ac(zaimu_id)
      if !bank_ac && (branch = find_bank(line))
        bank_ac = BankAccount.find(:first,
                   :conditions => ['bank_branch_id = ? AND ac_type = ? AND ac_number = ?',
                                   branch.id, line[33].to_i, line[34].to_i])
        if bank_ac && bank_ac['id'] != zaimu_id
          puts "dup: id = #{bank_ac['id']} and #{zaimu_id}"
        end
        if !bank_ac
          bank_ac = BankAccount.new()
          bank_ac.zaimu_ac = zaimu_id
          up = true
        end
      end
    
      if bank_ac
        # 口座の更新
        branch = find_bank(line) if !branch
        if branch && (bank_ac.bank_branch_id != branch.id ||
                      bank_ac.ac_type != line[33].to_i ||
                      bank_ac.ac_number != line[34].to_i ||
                      bank_ac.kana_name != line[35].strip)
          up = true
          bank_ac.bank_branch = branch
          bank_ac.ac_type = line[33]
          bank_ac.ac_number = line[34]
          bank_ac.kana_name = line[35].strip
        end

        if !bank_ac.partner
          up = true
          bank_ac.partner = create_partner(line)
        end
      
        bank_ac.save! if up
      else
        # 口座のない取引先は、初回のみ作る。
        # 運用開始したらいらない。
        # create_partner(line)
      end
    }
  } # of transaction
end


puts "Start at #{Time.now}"
import_partners()
puts "Finished at #{Time.now}"
