
# 部門、ユーザ（社員）のインポート
# 財務大将から


require File.dirname(__FILE__) + '/import_common.rb'
require 'csv'


def import_control_divisions
  data = sjis2utf8(File.read('control_divisions.csv'))
  ControlDivision.transaction {
    idx = 0
    CSV::Reader.parse(data) {|line|
      idx += 1; next if idx == 1
      raise if line[0].to_i == 0
      r = ControlDivision.find(:first, :conditions=>["id = ?", line[0].to_i]) || 
          ControlDivision.new(:id => line[0].to_i)
      r.name = line[1]
      r.save!
    }
  }
end

# 会計部門
def import_divisions
  data = sjis2utf8(File.read(DATA_DIR + '/bumon.csv'))
  Division.transaction {
    idx = 0
    CSV::Reader.parse(data) {|line|
      idx += 1; next if idx == 1
      next if line[0].to_i != 0 && line[0].to_i != 1
      r = find_division_by_code(line[1].strip) ||
              Division.new(
                  :division_code => line[1].strip,
                  :purchase_active => 1)
      r.attributes = {
        :name => line[2],
        :kana_name => (line[3] || ""),
        :end_day => line[5],
      }
      r.save!
    }
  }
end


# 社員マスタの取り込み。
# 財務大将の社員マスタでは所属部署がないので、ITFlowから取り込む。
def import_workers
  data = sjis2utf8 File.read('SyainMST.csv')
  
  User.transaction {
    idx = 0
    CSV::Reader.parse(data) {|line|
      idx += 1; next if idx <= 4
      raise if line[0].to_i == 0
      r = User.find_by_worker_code(line[0].to_i)
      if !r
        next if line[21].to_i != 0 # activeでないのをわざわざ追加しない
        r = User.new(:worker_code => line[0].to_i)
      end
      r.attributes = {
        :name => line[1],
        :kana_name => line[2],
        :email => line[6],
        :active => (line[21].to_i == 0 ? 1 : 0)
      }

      # 同一人物で社員コードが変わることがある。何だそれ。
      if r.email
        s = User.find :first,
                      :conditions => ['email = ? AND worker_code <> ?',
                                      r.email, r.worker_code]
        if s
          if s.active != 0
            if r.active == 0
              r.email = nil
            else
              puts "skip: (#{r.worker_code} #{r.name}) <=> (#{s.worker_code} #{s.name})"
              next
            end
          else
            # メールアドレスがリサイクルされている。えーっ！
            s.email = nil
            s.save!
          end
        end
      end
      r.save!
      
      # 部門を登録する。
      if line[3] && line[3].strip != ''
        ud = UsersDivision.find(:first,
                      :conditions => ['user_id = ? AND division_id = ?',
                                      r.id, find_division_by_code(line[3]).id])
        if !ud
          # 対応するレコードがないときはおそらく異動→既存のを消す
          UsersDivision.delete_all ['user_id = ?', r.id]
          ud = UsersDivision.new(:user => r,
                                 :division => find_division_by_code(line[3]))
        end
      
        # 以前のデフォルトをいったんクリアする
        UsersDivision.find_by_sql [<<EOF, r.id]
UPDATE users_divisions SET default_pair = 0 WHERE user_id = ?
EOF
        ud.default_pair = 1
        ud.save!
      end
    }
  } # of transaction
end

#import_control_divisions()
import_divisions()
import_workers()
