
# 下期予算をインポートする。10-3月のみ取り込む

require 'csv'
require './import_common.rb'

establish_connection 'import_yosan.log'

require '../app/models/account_title'
require '../app/models/control_division'
require '../app/models/division'
require "../app/models/kubun"
require "../app/models/general_ledger"
require "../app/models/work_action"
require "../app/models/user"

KUBUN_NAME = '2007_autumn_budget'

def add_kubun
  kubun = Kubun.new :name => KUBUN_NAME
  kubun.save!
end

# 予算データの削除
def clear_estimate
  kubun = Kubun.find_by_name KUBUN_NAME
  raise if !kubun
  
  GeneralLedger.transaction {
    GeneralLedger.delete_all ["kubun_id = ? and date >= '2007/04/01'", kubun.id]
    WorkAction.delete_all "id NOT IN (SELECT work_action_id FROM general_ledger WHERE work_action_id IS NOT NULL)"
=begin
動くが遅すぎ
DELETE FROM work_actions w 
    WHERE EXISTS (select * from work_actions b
                left join general_ledger on b.id = general_ledger.work_action_id 
                where w.id = b.id and general_ledger.work_action_id is null)
=end
  }
end



MON_COL = 8 # 金額の列番号 = I列
DIV_COL = 3 # 部門コード
ITEM_COL = 5 # 科目コード

def import_yosan2007_autumn
  GeneralLedger.transaction {
    ['2007autumn-jigyouhi.csv', '2007autumn-shokeihi.csv', '2007autumn-jinkenhi.csv', '2007autumn-lease.csv'].each {|filename|
      data = sjis2utf8(File.read(filename))
      
      kubun = Kubun.find_by_name KUBUN_NAME
      raise if !kubun

      idx = 0
      days = []
      CSV::Reader.parse(data) {|line|
        idx += 1
        if idx == 1
          days = (0..11).collect {|i| line[MON_COL + i]}
          next
        end

        if line[DIV_COL].to_i != 0 && line[ITEM_COL].to_i != 0
          div =  find_division_by_code(line[DIV_COL]) || raise("div = '#{line[DIV_COL]}', ac = #{line[ITEM_COL]}")
          ac = find_account_title(line[ITEM_COL].to_i, line[ITEM_COL + 1].to_i) || raise("ac = #{line[ITEM_COL]}/#{line[ITEM_COL + 1]}")

          wa = WorkAction.new :name => (line[0] || "") + (line[1] || "") + (line[2].to_s != '' ? " / #{line[2]}" : '')
          wa.save!

          (0..11).each {|i|
            gl = GeneralLedger.new(
                 :date => days[i],
                 :kubun => kubun,
                 :division => div,
                 :account_title => ac,
                 :work_action => wa,
                 :amount => (line[MON_COL + i].to_f + 0.5).to_i * 1000,
                 :remarks => wa.name)
            gl.save!
          }
        end
      }
    }
  }
end

# add_kubun
puts Time.now
clear_estimate
puts "cleared. #{Time.now}"
import_yosan2007_autumn
puts Time.now
