
# 2008年度予算をCSVファイルからインポートする。

require File.dirname(__FILE__) + '/import_common'
require 'csv'

KUBUN_NAME = '2008fy_budget'

def add_kubun
  if !Kubun.find_by_name(KUBUN_NAME)
    kubun = Kubun.new :name => KUBUN_NAME,
                      :long_name => '2008FY通期予算',
                      :active => 1
    kubun.save!
  end
end

# 予算データの削除
def clear_estimate
  kubun = Kubun.find_by_name KUBUN_NAME
  raise if !kubun
  
  GeneralLedger.transaction {
    GeneralLedger.delete_all ["kubun_id = ?", kubun.id]
    WorkAction.delete_all "id NOT IN (SELECT work_action_id FROM general_ledger WHERE work_action_id IS NOT NULL)"
  }
end


COLS = [ {
  :desc => 2,
  :mdiv => 0,     # 申請部門
  :div => 4,      # 計上部門
  :ac_item => 6,  # 勘定科目
  :amounts => 9}, # 金額 <j列>
{
  :desc => 0,
  :div => 3,
  :ac_item => 5,
  :amounts => 8}
]

FILES = {
  '2008fy-jigyouhi.csv' => 0,
  '2008fy-shokeihi.csv' => 1,
  '2008fy-lease.csv' => 1,
  '2008fy-jigyou2.csv' => 0,
}


def import_operating_cost
  GeneralLedger.transaction {
    kubun = Kubun.find_by_name KUBUN_NAME
    raise if !kubun

    FILES.each {|filename, col_type|
      col = COLS[col_type]
      data = sjis2utf8(File.read(filename))

      idx = 0
      days = []
      CSV::Reader.parse(data) {|line|
        idx += 1
        if idx == 1
          days = (0..11).collect {|i| line[col[:amounts] + i]}
          next
        end

        mdiv_val = col[:mdiv] ? line[col[:mdiv]] : line[col[:div]]
        div_val = line[col[:div]]
        item_val = line[col[:ac_item]]

        if mdiv_val.to_i != 0 || div_val.to_i != 0 || item_val.to_i != 0
          if mdiv_val.to_i == 0 || div_val.to_i == 0 || item_val.to_i == 0
            raise "0 error: " + line.to_s
          end
          mdiv = find_division_by_code(mdiv_val) || raise("mdiv = '#{mdiv_val}', ac = #{item_val}")
          div =  find_division_by_code(div_val) || raise("div = '#{div_val}', ac = #{item_val}")
          ac = find_account_title(item_val.to_i, line[col[:ac_item] + 1].to_i) || raise("ac = #{item_val}/#{line[col[:ac_item] + 1]}")

          wa = WorkAction.new :name => line[col[:desc]].to_s,
                              :partners => line[col[:desc] + 1].to_s,
                              :division_id => mdiv.id
          wa.save!

          (0..11).each {|i|
            if line[col[:amounts] + i].to_i != 0
              gl = GeneralLedger.new(
                      :date => days[i],
                      :kubun => kubun,
                      :division => div,
                      :account_title => ac,
                      :work_action => wa,
                      :amount => (line[col[:amounts] + i].to_f + 0.5).to_i * 1000 * ac.sign,
                      :remarks => wa.name)
              gl.save!
            end
          }
        end
      }
    }
  }
end


puts Time.now
add_kubun
clear_estimate
puts "cleared. #{Time.now}"
import_operating_cost
puts Time.now
